import TypeUtils from "./utils/type";

import ActionsRunMultiple from "./actions/runMultiple";

import ActionsFormsSubmitV1 from "./actions/forms/submit";

import ActionsHttpPostV1 from "./actions/http/post";
import ActionsHttpPatchV1 from "./actions/http/patch";
import ActionsHttpDeleteV1 from "./actions/http/delete";

import ActionsDialogsAlert from "./actions/dialogs/alert";
// import ActionsDialogsConfirm from "./actions/dialogs/confirm"
import ActionsDialogsShow from "./actions/dialogs/show";
import ActionsDialogsOpen from "./actions/dialogs/open";
import ActionsDialogsClose from "./actions/dialogs/close";
import ActionsDialogsOptions from "./actions/dialogs/options";
import ActionsDialogsNotification from "./actions/dialogs/notification";
import ActionsDialogsOauth from "./actions/dialogs/oauth";

import ActionsSnackbarsAlert from "./actions/snackbars/alert";
// import ActionsSnackbarsConfirm from "./actions/snackbars/confirm"
import ActionsSnackbarsSelect from "./actions/snackbars/select";

import ActionsSheetsSelect from "./actions/sheets/select";

import ActionsWindowsClose from "./actions/windows/close";
import ActionsWindowsCloseAll from "./actions/windows/closeAll";
import ActionsWindowsOpen from "./actions/windows/open";
import ActionsWindowsOpenWeb from "./actions/windows/openWeb";
import ActionsWindowsReload from "./actions/windows/reload";
import ActionsWindowsCloseWithReload from "./actions/windows/closeWithReload";
import ActionsWindowsRefreshState from "./actions/windows/refreshState";

import ActionsPanelsScrollToBottom from "./actions/panels/scrollToBottom";
import ActionsPanelsScrollTo from "./actions/panels/scrollTo";

import ActionsWsPush from "./actions/ws/push";
import ActionsCablesPush from "./actions/cables/push";

import ActionsTimeoutsSet from "./actions/timeouts/set";

import ActionsAuthSaveCsrfToken from "./actions/auth/saveCsrfToken";
import ActionsAuthRestart from "./actions/auth/restart";
import ActionsCreditCard from "./actions/auth/creditCard";

import ActionsAnalyticsLogEvent from "./actions/analytics/logEvent";

const actions = {
  runMultiple: ActionsRunMultiple,

  "forms/submit": ActionsFormsSubmitV1,

  "http/post": ActionsHttpPostV1,
  "http/patch": ActionsHttpPatchV1,
  "http/delete": ActionsHttpDeleteV1,

  "dialogs/alert": ActionsDialogsAlert,
  // "dialogs/confirm": ActionsDialogsConfirm,
  "dialogs/show": ActionsDialogsShow,
  "dialogs/open": ActionsDialogsOpen,
  "dialogs/close": ActionsDialogsClose,
  "dialogs/options": ActionsDialogsOptions,
  "dialogs/notification": ActionsDialogsNotification,
  "dialogs/oauth": ActionsDialogsOauth,

  "snackbars/alert": ActionsSnackbarsAlert,
  // "snackbars/confirm": ActionsSnackbarsConfirm,
  "snackbars/select": ActionsSnackbarsSelect,

  "sheets/select": ActionsSheetsSelect,

  "windows/close": ActionsWindowsClose,
  "windows/closeAll": ActionsWindowsCloseAll,
  "windows/open": ActionsWindowsOpen,
  "windows/openWeb": ActionsWindowsOpenWeb,
  "windows/reload": ActionsWindowsReload,
  "windows/closeWithReload": ActionsWindowsCloseWithReload,
  "windows/refreshState": ActionsWindowsRefreshState,

  "panels/scrollToBottom": ActionsPanelsScrollToBottom,
  "panels/scrollTo": ActionsPanelsScrollTo,

  "timeouts/set": ActionsTimeoutsSet,

  "ws/push": ActionsWsPush,
  "cables/push": ActionsCablesPush,

  "auth/saveCsrfToken": ActionsAuthSaveCsrfToken,
  "auth/restart": ActionsAuthRestart,
  "auth/creditCard": ActionsCreditCard,

  "analytics/logEvent": ActionsAnalyticsLogEvent
};

export default class Action {
  static execute(spec, target, component, params = {}) {
    if (!TypeUtils.isObject(spec)) {
      return;
    }

    const name = spec.action;
    if (TypeUtils.isString(name)) {
      if (name.startsWith("component/")) {
        this.executeLocal(name, spec, component);
      } else {
        this.executeGlobal(name, spec, component, params);
      }

      // const actionName = name.replace(/-v1$/, "");

      // try {
      //   const action = new actions[actionName]();
      //   const logDisabled = action.logDisabled && action.logDisabled();
      //   if (!logDisabled) {
      //     console.log(`Executing "${actionName}"`);
      //   }
      //   action.execute(spec, null, component, params);
      // } catch (e) {
      //   console.log(`Failed executing command: "${e.message}"`);
      // }
    } else {
      console.log(`Invalid action: "${name}"`);
    }
  }

  static executeLocal(name, spec, component) {
    const methodName = name.replace(/^component\//, "");
    const componentName = Utils.app.vueName(component);
    console.log(
      `Executing component action on ${componentName}: ${methodName}`
    );
    component[`action_${methodName}`](spec);
  }

  static executeGlobal(name, spec, component, params) {
    const actionName = name.replace(/-v1$/, "");

    try {
      const action = new actions[actionName]();
      const logDisabled = action.logDisabled && action.logDisabled();
      if (!logDisabled) {
        console.log(`Executing "${actionName}"`);
      }
      action.execute(spec, null, component, params);
    } catch (e) {
      console.log(
        "Failed executing command",
        actionName,
        `Error: '${e.message}'`
      );
    }
  }

  static handleResponse(response, component) {
    if (response.header || response.body || response.footer) {
      Utils.http.forceComponentUpdate(() => {
        window.vueApp.page = response;
      });
    }

    window.vueApp.temp.analytics = response.analytics;
    GLib.action.execute(response.onResponse, null, component);
    window.vueApp.temp.analytics = null;
  }
}

window.Action = Action;
