import Vue from "vue";

export default class {
  static saveInitialContent(content) {
    // History only handles document's scroll position. It doesn't handle our body div's position
    this.bodyScrollTops = [];
    // This differs from window.history.length in that this only counts internal navigation
    this.navigationCount = 0;
    // Set initial state so we can go back to it
    window.history.replaceState(this._data(content), "", window.location.href);
  }

  static get _pageBody() {
    return document.getElementById("page_body");
  }

  static _data(content) {
    return { content: content, navigationCount: this.navigationCount };
  }

  static pushPage(content, url) {
    // Save scroll position of the current page when navigating to the next page
    this.bodyScrollTops[this.navigationCount] = this._pageBody.scrollTop;

    this.navigationCount++;

    // Save data of the next page (page getting pushed)
    window.history.pushState(this._data(content), "", url);
  }

  static resetScroll() {
    this._pageBody.scrollTop = 0;
  }

  static restoreOnBackOrForward() {
    const vm = this;
    window.onpopstate = event => {
      // Save scroll position of the current page when navigating through back/forward button
      this.bodyScrollTops[this.navigationCount] = this._pageBody.scrollTop;

      const data = event.state;
      data.content.__poppedState = true;
      window.vueApp.page = data.content;
      vm.navigationCount = data.navigationCount;

      Vue.nextTick(() => {
        const scrollTop = this.bodyScrollTops[this.navigationCount];
        this._pageBody.scrollTop = scrollTop;
      });
    };
  }

  static isPoppedState(page) {
    return page.__poppedState == true;
  }

  static updatePage(url = window.location.href) {
    window.history.replaceState(this._data(window.vueApp.page), "", url);
  }

  static back() {
    if (this.navigationCount > 0) {
      window.history.back();
      return true;
    } else {
      console.log("Reached beginning of navigation stack");
      return false;
    }
  }
}
