import Vue from "vue";
import Dialog from "../nav/dialog";
import Sheet from "../nav/sheet";
import Snackbar from "../nav/snackbar";
import Vuetify from "vuetify";

export default class {
  static dialog(properties, target) {
    if (!this.stack) {
      this.stack = [];
    }

    // https://css-tricks.com/creating-vue-js-component-instances-programmatically/
    const DialogClass = Vue.extend(Dialog);
    const instance = new DialogClass({
      vuetify: new Vuetify({
        theme: {
          themes: Utils.settings.themes
        }
      }),
      propsData: {
        spec: properties,
        stack: this.stack,
        target: target,
        vueApp: window.vueApp
      }
    });
    // instance.stack = this.stack;

    const placeholder = document.body.appendChild(
      document.createElement("div")
    );
    instance.$mount(placeholder);
  }

  static closeDialog(properties, target, component) {
    if (!this.stack) {
      this.stack = [];
    }

    Utils.type.ifObject(this.stack.last(), dialog => {
      dialog.close();
    });

    Utils.type.ifObject(properties["onClose"], it => {
      Action.execute(it, target, component);
    });
  }

  static get sheet() {
    return LaunchSheet;
  }

  static get snackbar() {
    return LaunchSnackbar;
  }

  static alert(message, target) {
    const properties = {
      message: message
    };
    const spec = Object.assign({}, properties, {
      buttons: [
        {
          text: "OK",
          onClick: properties.onClose
        }
      ]
    });
    this.dialog(spec, target);
  }
}

class LaunchSheet {
  static open(properties) {
    const SheetClass = Vue.extend(Sheet);
    const instance = new SheetClass({
      vuetify: new Vuetify({
        theme: {
          themes: Utils.settings.themes
        }
      }),
      propsData: { spec: properties }
    });
    const placeholder = document.body.appendChild(
      document.createElement("div")
    );
    instance.$mount(placeholder);
    // instance.show = true
    return instance;
  }

  static confirm(properties, onConfirm) {
    const spec = Object.assign({}, properties, {
      buttons: [
        {
          text: "Confirm",
          onClickFunction: onConfirm
        }
      ]
    });
    this.open(spec);
  }

  static alert(message, buttons) {
    const spec = { message: message, buttons: buttons };
    this.open(spec);
  }
}

class LaunchSnackbar {
  static open(properties) {
    const SnackbarClass = Vue.extend(Snackbar);
    const instance = new SnackbarClass({
      vuetify: window.vueApp.vuetify,
      propsData: { spec: properties, vueApp: window.vueApp }
    });

    const placeholder = document.body.appendChild(
      document.createElement("div")
    );
    instance.$mount(placeholder);
    instance.show = true;
    return instance;
  }

  static error(message) {
    this.open({ message: message, styleClasses: ["error"] });
  }

  static indicator(message) {
    const instance = this.open({ message: message, timeout: -1 });
    instance.indicator = true;
  }
}
